part of '../../engine.dart';

/// Manages booting, creating subsystems, and running the logic and render loop.
class Game {
  /// Timestamp function.
  @protected
  final Clock clock;

  /// Which renderer is being used.
  @protected
  final Renderer renderer;

  /// Internal event scheduling system.
  Scheduler _scheduler;

  /// Creates a new [Game].
  ///
  /// May provide the specified [width] and [height] in pixels.
  Game(
    this.renderer, {
    this.clock = defaultClock,
    int width = 800,
    int height = 600,
  })  : assert(width != null),
        assert(height != null),
        assert(renderer != null),
        _width = width,
        _height = height {
    _scheduler = createScheduler();
  }

  /// Current game width in pixels.
  int get width => _width;
  int _width;

  /// Current game height in pixels.
  int get height => _height;
  int _height;

  /// Creates a [Scheduler] for this game instance.
  Scheduler createScheduler() => new Scheduler();

  /// Invokes the core game loop, with the current [time].
  @protected
  void update(num time) {
    _scheduler.update(time);
  }
}
