part of '../../geometry.dart';

/// An immutable 2-dimensional coordinate pair.
///
/// **NOTE**: This is intentionally different than `dart:math:Point`.
@Immutable()
class Point {
  /// X-axis coordinate.
  final int x;

  /// Y-axis coordinate.
  final int y;

  const Point(this.x, this.y) : assert(x != null), assert(y != null);

  @override
  bool operator==(Object o) => o is Point && x == o.x && y == o.y;

  @override
  int get hashCode => hash2(x, y);

  /// Returns the distance between `this` and [other].
  double distanceTo(Point other) {
    final dx = x - other.x;
    final dy = y - other.y;
    return math.sqrt(dx * dx + dy * dy);
  }

  /// Returns the sum of adding coordinates of `this` point and [o].
  Point operator+(Point o) => new Point(x + o.x, y + o.y);

  /// Returns the difference of subtracting coordinates of `this` point and [o].
  Point operator-(Point o) => new Point(x - o.x, y - o.y);

  /// Returns `this` point's coordinates multiplied by [o]'s.
  Point operator*(Point o) => new Point(x * o.x, y * o.y);

  @override
  String toString() {
    if (assertionsEnabled) {
      return '$Point {$x, $y}';
    }
    return super.toString();
  }
}
