import 'package:dazer/foundation.dart';
import 'package:dazer/geometry.dart';
import 'package:test/test.dart';

void main() {
  test('should create a line from two points', () {
    const line = const Line.fromPoints(const Point(0, 0), const Point(0, 2));
    expect(line.start, const Point(0, 0));
    expect(line.end, const Point(0, 2));
  });

  test('should create a line from four coordinates', () {
    final line = new Line(0, 0, 0, 2);
    expect(line.start, const Point(0, 0));
    expect(line.end, const Point(0, 2));
  });

  test('should be equal/hash same to a similar line', () {
    final lineA = new Line(0, 0, 0, 2), lineB = new Line(0, 0, 0, 2);
    expect(lineA, isNot(same(lineB)));
  });

  test('should compute the angle of the line', () {
    expect(toDegrees(new Line(0, 0, 0, 2).angle), 90);
  });

  test('should compute the slope of the line', () {
    expect(new Line(0, 0, 1, 2).slope, 2);
  });

  test('should compute the width and height of the line', () {
    final line = new Line(0, 0, 1, 2);
    expect(line.width, 1);
    expect(line.height, 2);
  });

  test('should compute top/bottom/left/right of the line', () {
    final line = new Line(0, 0, 1, 2);
    expect(line.left, 0);
    expect(line.top, 0);
    expect(line.right, 1);
    expect(line.bottom, 2);
  });

  test('should have a descriptive toString()', () {
    expect('${new Line(0, 0, 1, 2)}', 'Line {0, 0} -> {1, 2}');
  }, skip: !assertionsEnabled);
}
